package dMartinez;


import gui.run.RunMenu;
import gui.run.RunMenuBar;
import gui.run.RunMenuItem;
import gui.run.RunJob;
import net.stocks.Quote;
import net.web.UrlUtils;

import javax.swing.*;
import java.awt.*;
import java.util.StringTokenizer;
import java.util.Vector;


public class StocksFrame extends
        gui.ClosableJFrame {
    public Vector quotes = new Vector();

    JLabel labels[];
    Vector list = new Vector();
    static StocksFrame sf;
    ListDemo ld;
    JPanel display = new JPanel();
    public Container c = getContentPane();

    public void init(Vector newQuotes) {
        ld = new ListDemo(sf);
        getQuotes(newQuotes);
        labels = new JLabel[quotes.size()];
        updatePanel();
        setContainerLayout(new BorderLayout());
        c.add(setJMenuBar(), BorderLayout.NORTH);
        c.add(display);
        pack();
        setVisible(true);
        startupdateThread();
    }

    private RunMenuBar setJMenuBar() {
        RunMenuBar mb = new RunMenuBar();
        mb.add(getFileMenu());
        mb.add(getEditMenu());
        return mb;
    }

    public static RunMenu getFileMenu() {
        RunMenu fileMenu = new RunMenu("[File");

        fileMenu.add(new RunMenuItem("[quit{control q}") {
            public void run() {
                System.exit(0);
            }
        });

        return fileMenu;
    }

    public RunMenu getEditMenu() {
        RunMenu fileMenu = new RunMenu("[Edit");

        fileMenu.add(new RunMenuItem("[customize{control c}") {
            public void run() {
                ld.createAndShowGUI();
            }
        });

        return fileMenu;
    }

    /**
     * This makes sure that the symbols enterd by the user is correct.
     *
     * @param symbol- stock symbol
     * @return good- if symbol is valid. Bad- if symbol is invalid.
     */
    public String checkInputSymbol(String symbol) {
        String validityOFSymbol;
        validityOFSymbol = checkSymbol(symbol);//we need to get the symbols checked before we
        //add them or remove them from the list
        if (validityOFSymbol.equals("good")) {
            return ("good");
        }
        return ("bad");
    }

    /**
     * The symbols that the user enters are checked before displaying the quotes.
     *
     * @param symbol- stock symbol
     * @return good- for a valid symbol. Bad-for an invalid symbol.
     */
    public String checkSymbol(String symbol) {
        String s = UrlUtils.getUrlVector(getUrlSource(symbol)).toString();

        StringTokenizer token = new StringTokenizer(s, "[], ");
        String st;
        int count = 0;//refers to the "N/A" string counts
        while (token.hasMoreTokens()) {
            st = token.nextToken();
            if (st.equals("N/A")) {
                count = count + 1;
            }
        }
        //if there are 4 or more N/A counts than the symbol is not valid
        if (count >= 4) {
            return "bad";
        }
        return "good";
    }

    /**
     * This is the URL source that generates quotes for any symbol
     *
     * @param symbol- Stock Symbol
     * @return - the Url's source code
     */
    public static String getUrlSource(String symbol) {
        return
                "http://216.109.124.128/download/javasoft.beans?SYMBOLS=" + symbol + "&format=sl";
    }

    public String[] quoteLineToStringVector(String pcs) {
        Vector stringVector =
                new Vector();
        String s = pcs;
        //separate the strings delimited by commas
        for (
                int i = s.indexOf(',');
                i >= 0;
                i = s.indexOf(',')) {
            stringVector.addElement(s.substring(0, i));
            s = s.substring(i + 1);
        }
        // add the last String left
        stringVector.addElement(s);
        String sa[] =
                new String[stringVector.size()];
        for (int i = 0; i < sa.length; i++)
            sa[i] = (String) stringVector.elementAt(i);
        return sa;
    }

    public void setList(Vector lis) {
        sf.list = new Vector();
        for (int x = 0; x < lis.size(); x++) {
            sf.list.addElement(lis.elementAt(x));
        }
    }

    public void updateDisplay() {
        quotes = new Vector();

        if (list.size() > 0) {
            getQuotes(list);
            setLabel();
        }
        if (list.size() == 0) {
            getQuotes(null);
        }
        for (int i = 0; i < quotes.size(); i++) {
            labels[i].setText((String) quotes.elementAt(i));
        }
        updatePanel();
    }

    public void updatePanel() {
        c.remove(display);
        display = new JPanel();
        display.setBackground(Color.cyan);
        display.setLayout(new GridLayout(0, 1));

        for (int i = 0; i < quotes.size(); i++) {
            String s =
                    (String) quotes.elementAt(i);
            quoteLineToStringVector(s);
            sf.labels[i] = new JLabel(s);
            sf.display.add(labels[i]);
        }
        c.add(display);
        sf.pack();

    }

    public void setLabel() {
        sf.labels = new JLabel[quotes.size()];
        for (int i = 0; i < labels.length; i++) {
            String s =
                    (String) quotes.elementAt(i);
            quoteLineToStringVector(s);
            sf.labels[i] = new JLabel(s);
        }
    }
    public void startupdateThread(){
        new RunJob(2){
            public void run(){
                updateDisplay();
            }
        };
    }
    public void paint(Graphics g) {
        super.paint(g);
        updateDisplay();
    }

    /**
     * This return quotes for a vector of stocks symbols. If l is null, a default vector
     * is used.
     *
     * @param l- a vector of stock symbols.
     * @return the quotes for all the symbols in l.
     */
    public Vector getQuotes(Vector l) {
        if (l == null) {
            Vector s = new Vector();
            s.addElement("a");
            s.addElement("open");
            s.addElement("xlnx");
            s.addElement("altr");
            s.addElement("mot");
            s.addElement("cy");
            s.addElement("crus");
            s.addElement("sfa");
            s.addElement("adbe");
            s.addElement("msft");
            s.addElement("sunw");
            s.addElement("lxk");
            s.addElement("AMZn");
            s.addElement("NT");
            return sf.quotes =
                    Quote.getUrl(Quote.makeQuoteURLString(s));
        }

        return sf.quotes = Quote.getUrl(Quote.makeQuoteURLString(l));
    }

    public static void main(String args[]) {
        sf = new StocksFrame();
        sf.init(null);
    }


}