package security;

import utils.PreferencesUtils;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import java.io.Serializable;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

/**
 * Copyright 2005, DocJava, Inc.
 * User: dlyon
 * Date: Oct 18, 2005
 * Time: 3:49:09 PM
 */
public class CodecBean implements Serializable {
    private Cipher cipher;
    private Key key;

    private static final String preferencesKey = "codecBean";

    public Cipher getCipher() {
        return cipher;
    }

    /**
     * Save into user prefs.
     */
    public void save(){
        PreferencesUtils pu = new PreferencesUtils(preferencesKey);
        pu.save(this);
    }

    /**
     * restore from user prefs.
     * @return  new <code>CodecBean</code> if there are none saved.
     */
    public static CodecBean restore(){
        PreferencesUtils pu = new PreferencesUtils(preferencesKey);
        Object o = pu.restore();
        if (o == null) return new CodecBean();
        if (o instanceof CodecBean) return (CodecBean)o;
        return new CodecBean();
    }
    /**
     * Use a <code>keyString</code> to set the key in the cipher to
     * obtain a deterministic codec.  Use the <I>DES</I> algorithm
     * for creating the key.
     * @param keyString
     * @throws NoSuchAlgorithmException
     */
    public void setKey(String keyString) throws NoSuchAlgorithmException {
        KeyGenerator generator = KeyGenerator.getInstance("DES");
        SecureRandom secureRandom = new SecureRandom();
        secureRandom.setSeed(keyString.hashCode());
        generator.init(secureRandom);
        key = generator.generateKey();
    }

    public void setCipher(Cipher cipher) {
        this.cipher = cipher;
    }

    public Key getKey() {
        return key;
    }

    public void setKey(Key key) {
        this.key = key;
    }
}
